@@ -1,6 +1,6 @@
 /*
-	acr38cmd.h: Emulated CCID reader commands for ACR38 non-CCID reader
-	Copyright (C) 2011-2012   Advanced Card Systems Ltd.
+	acr38cmd.c: Emulated CCID reader commands for ACR38 non-CCID reader
+	Copyright (C) 2011-2013   Advanced Card Systems Ltd.
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Lesser General Public
@@ -492,7 +492,7 @@ RESPONSECODE ACR38_SetCardVoltage(unsigned int reader_index,
 	unsigned char RxBuffer[], unsigned int *RxLength)
 {
 	RESPONSECODE return_value = IFD_SUCCESS;
-	unsigned char cardVoltage;
+	BYTE cardVoltage;
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 
 	if ((TxBuffer != NULL) && (TxLength >= 1))
@@ -520,12 +520,12 @@ RESPONSECODE ACR38_SetCardType(unsigned int reader_index,
 	unsigned char RxBuffer[], unsigned int *RxLength)
 {
 	RESPONSECODE return_value = IFD_SUCCESS;
-	unsigned long cardType;
+	DWORD cardType;
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 
 	if ((TxBuffer != NULL) && (TxLength >= sizeof(cardType)))
 	{
-		cardType = *((unsigned long *) TxBuffer);
+		cardType = *((PDWORD) TxBuffer);
 		switch (cardType)
 		{
 			case CARD_TYPE_MCU_AUTO:
