@@ -1,7 +1,7 @@
 /*
     ccid.c: CCID common code
     Copyright (C) 2003-2005   Ludovic Rousseau
-    Copyright (C) 2009-2012   Advanced Card Systems Ltd.
+    Copyright (C) 2009-2013   Advanced Card Systems Ltd.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -611,6 +611,31 @@ int ccid_open_hack_post(unsigned int reader_index)
 				ccid_descriptor->isSamSlot = 1;	// SAM
 			break;
 
+		case ACS_ACR1281U_K_DUAL_READER:
+		case ACS_ACR1281U_K_1S_DUAL_READER:
+		case ACS_ACR1281U_K_4S_DUAL_READER:
+			if (ccid_descriptor->bCurrentSlotIndex == 1)
+			{
+				ccid_descriptor->dwFeatures = 0x000204BA;	// ICC
+				ccid_descriptor->dwMaxDataRate = 600000;
+			}
+			else if (ccid_descriptor->bCurrentSlotIndex > 1)
+			{
+				ccid_descriptor->dwFeatures = 0x000204BA;	// SAM
+				ccid_descriptor->dwMaxDataRate = 172043;
+				ccid_descriptor->isSamSlot = 1;
+			}
+			break;
+
+		case ACS_ACR123_3S_READER:
+			if (ccid_descriptor->bCurrentSlotIndex > 0)
+			{
+				ccid_descriptor->dwFeatures = 0x000204BA;	// SAM
+				ccid_descriptor->dwMaxDataRate = 125000;
+				ccid_descriptor->isSamSlot = 1;
+			}
+			break;
+
 		default:
 			break;
 	}
