@@ -1,7 +1,7 @@
 /*
     ccid_usb.c: USB access routines using the libusb library
     Copyright (C) 2003-2009   Ludovic Rousseau
-    Copyright (C) 2009-2012   Advanced Card Systems Ltd.
+    Copyright (C) 2009-2013   Advanced Card Systems Ltd.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -468,7 +468,8 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 						else if ((ACS_ACR1281_DUAL_READER_QPBOC == readerID) ||
 							(ACS_ACR1281_DUAL_READER_BSI == readerID) ||
 							(ACS_ACR1281_1S_PICC_READER == readerID) ||
-							(ACS_ACR1251_1S_CL_READER == readerID))
+							(ACS_ACR1251_1S_CL_READER == readerID) ||
+							(ACS_ACR1251U_C == readerID))
 						{
 							// the CCID interfaces are 0 and 1
 							interface_number = static_interface - 1;
@@ -854,7 +855,8 @@ again:
 
 					// The 2nd interface (composite device) is a SAM slot
 					if ((readerID == ACS_ACR1281_1S_PICC_READER) ||
-						(readerID == ACS_ACR1251_1S_CL_READER))
+						(readerID == ACS_ACR1251_1S_CL_READER) ||
+						(readerID == ACS_ACR1251U_C))
 					{
 						if (interface == 1)
 							usbDevice[reader_index].ccid.isSamSlot = TRUE;
