@@ -1,6 +1,7 @@
 /*
     commands.c: Commands sent to the card
     Copyright (C) 2003-2004   Ludovic Rousseau
+    Copyright (C) 2010-2012   Advanced Card Systems Ltd.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -28,15 +29,14 @@
 #include <ifdhandler.h>
 #include <reader.h>
 
+#include "config.h"
 #include "misc.h"
 #include "commands.h"
 #include "openct/proto-t1.h"
 #include "ccid.h"
 #include "defs.h"
 #include "ccid_ifdhandler.h"
-#include "config.h"
 #include "debug.h"
-#include "ccid_usb.h"
 
 /* All the pinpad readers I used are more or less bogus
  * I use code to change the user command and make the firmware happy */
@@ -48,7 +48,9 @@
 #define BOGUS_SCM_FIRMWARE_FOR_dwMaxCCIDMessageLength
 
 #define max( a, b )   ( ( ( a ) > ( b ) ) ? ( a ) : ( b ) )
+#ifndef offsetof
 #define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)
+#endif
 
 // Fix problem using pcsc-lite 1.6.x header files
 #ifndef IFD_ERROR_INSUFFICIENT_BUFFER
@@ -162,7 +164,7 @@ RESPONSECODE CmdPowerOn(unsigned int reader_index, unsigned int * nlength,
 		}
 
 		DEBUG_INFO_XXD("Data Block: ", tmp, r);
-		if (*nlength > r-1)
+		if ((int)*nlength > r-1)
 			*nlength = r-1;
 		memcpy(buffer, tmp+1, *nlength);
 
@@ -283,7 +285,7 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 	unsigned char TxBuffer[], unsigned int TxLength,
 	unsigned char RxBuffer[], unsigned int *RxLength)
 {
-	unsigned char cmd[11+14+CMD_BUF_SIZE];
+	unsigned char cmd[11+14+TxLength];
 	unsigned int a, b;
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 	int old_read_timeout;
@@ -297,14 +299,6 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 	cmd[9] = 0;
 	cmd[10] = 0;	/* bPINOperation: PIN Verification */
 
-	/* 19 is the size of the PCSCv2 PIN verify structure
-	 * The equivalent CCID structure is only 14-bytes long */
-	if (TxLength > 19+CMD_BUF_SIZE) /* command too large? */
-	{
-		DEBUG_INFO3("Command too long: %d > %d", TxLength, 19+CMD_BUF_SIZE);
-		return IFD_NOT_SUPPORTED;
-	}
-
 	if (TxLength < 19+4 /* 4 = APDU size */)	/* command too short? */
 	{
 		DEBUG_INFO3("Command too short: %d < %d", TxLength, 19+4);
@@ -376,7 +370,7 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 			TxLength - offsetof(PIN_VERIFY_STRUCTURE, abData));
 
 		/* Create T=1 block */
-		ret = t1_build(&((get_ccid_slot(reader_index))->t1),
+		(void)t1_build(&((get_ccid_slot(reader_index))->t1),
 			sdata, 0, T1_I_BLOCK, &sbuf, NULL);
 
 		/* Increment the sequence numbers  */
@@ -447,12 +441,13 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 	ret = CCID_Receive(reader_index, RxLength, RxBuffer, NULL);
 
 	/* T=1 Protocol Management for a TPDU reader */
-	if ((IFD_SUCCESS == ret)
-		&& (SCARD_PROTOCOL_T1 == ccid_descriptor->cardProtocol)
+	if ((SCARD_PROTOCOL_T1 == ccid_descriptor->cardProtocol)
 		&& (CCID_CLASS_TPDU == (ccid_descriptor->dwFeatures & CCID_CLASS_EXCHANGE_MASK)))
 	{
 		/* timeout and cancel cases are faked by CCID_Receive() */
-		if (2 == *RxLength)
+		if ((2 == *RxLength)
+			/* the CCID command is rejected or failed */
+		   || (IFD_SUCCESS != ret))
 		{
 			/* Decrement the sequence numbers since no TPDU was sent */
 			get_ccid_slot(reader_index)->t1.ns ^= 1;
@@ -482,7 +477,7 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 	unsigned char TxBuffer[], unsigned int TxLength,
 	unsigned char RxBuffer[], unsigned int *RxLength)
 {
-	unsigned char cmd[11+19+CMD_BUF_SIZE];
+	unsigned char cmd[11+19+TxLength];
 	unsigned int a, b;
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 	int old_read_timeout;
@@ -499,14 +494,6 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 	cmd[9] = 0;
 	cmd[10] = 1;	/* bPINOperation: PIN Modification */
 
-	/* 24 is the size of the PCSC PIN modify structure
-	 * The equivalent CCID structure is only 18 or 19-bytes long */
-	if (TxLength > 24+CMD_BUF_SIZE) /* command too large? */
-	{
-		DEBUG_INFO3("Command too long: %d > %d", TxLength, 24+CMD_BUF_SIZE);
-		return IFD_NOT_SUPPORTED;
-	}
-
 	if (TxLength < 24+4 /* 4 = APDU size */) /* command too short? */
 	{
 		DEBUG_INFO3("Command too short: %d < %d", TxLength, 24+4);
@@ -596,7 +583,7 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 			TxLength - offsetof(PIN_MODIFY_STRUCTURE, abData));
 
 		/* Create T=1 block */
-		ret = t1_build(&((get_ccid_slot(reader_index))->t1),
+		(void)t1_build(&((get_ccid_slot(reader_index))->t1),
 			sdata, 0, T1_I_BLOCK, &sbuf, NULL);
 
 		/* Increment the sequence numbers  */
@@ -644,7 +631,7 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 		/* copy to the CCID block 'verbatim' */
 		cmd[a] = TxBuffer[b];
 		a++;
- 	}
+	}
 
 #ifdef BOGUS_PINPAD_FIRMWARE
 	if ((SPR532 == ccid_descriptor->readerID)
@@ -667,19 +654,20 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 
 	if (WritePort(reader_index, a, cmd) != STATUS_SUCCESS)
 	{
- 		ret = IFD_COMMUNICATION_ERROR;
+		ret = IFD_COMMUNICATION_ERROR;
 		goto end;
 	}
 
- 	ret = CCID_Receive(reader_index, RxLength, RxBuffer, NULL);
+	ret = CCID_Receive(reader_index, RxLength, RxBuffer, NULL);
 
 	/* T=1 Protocol Management for a TPDU reader */
-	if ((IFD_SUCCESS == ret)
-		&& (SCARD_PROTOCOL_T1 == ccid_descriptor->cardProtocol)
+	if ((SCARD_PROTOCOL_T1 == ccid_descriptor->cardProtocol)
 		&& (CCID_CLASS_TPDU == (ccid_descriptor->dwFeatures & CCID_CLASS_EXCHANGE_MASK)))
 	{
 		/* timeout and cancel cases are faked by CCID_Receive() */
-		if (2 == *RxLength)
+		if ((2 == *RxLength)
+			/* the CCID command is rejected or failed */
+			|| (IFD_SUCCESS != ret))
 		{
 			/* Decrement the sequence numbers since no TPDU was sent */
 			get_ccid_slot(reader_index)->t1.ns ^= 1;
@@ -775,6 +763,7 @@ again:
 
 	if (length_out < STATUS_OFFSET+1)
 	{
+		free(cmd_out);
 		DEBUG_CRITICAL2("Not enough data received: %d bytes", length_out);
 		return_value = IFD_COMMUNICATION_ERROR;
 		goto end;
@@ -913,7 +902,7 @@ RESPONSECODE CmdGetSlotStatus(unsigned int reader_index, unsigned char buffer[])
 
 again_status:
 		/* SlotStatus */
-		r = ControlUSB(reader_index, 0xA1, 0xA0, 0, status, sizeof(status)); 
+		r = ControlUSB(reader_index, 0xA1, 0xA0, 0, status, sizeof(status));
 
 		/* we got an error? */
 		if (r < 0)
@@ -1006,20 +995,12 @@ again_status:
 		return IFD_COMMUNICATION_ERROR;
 	}
 
-	if (buffer[STATUS_OFFSET] & CCID_COMMAND_FAILED)
-	{
-#ifdef O2MICRO_OZ776_PATCH
-		/* the O2MICRO OZ 776 reader sends card absent or mute errors
-		 * when no card is inserted */
-		if (! (((OZ776 == ccid_descriptor->readerID)
-			|| (OZ776_7772 == ccid_descriptor->readerID))
-			&& (buffer[ERROR_OFFSET] == 0xFE)))
-#endif
-		ccid_error(buffer[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
-
+	if ((buffer[STATUS_OFFSET] & CCID_COMMAND_FAILED)
 		/* card absent or mute is not an communication error */
-		if (buffer[ERROR_OFFSET] != 0xFE)
-			return_value = IFD_COMMUNICATION_ERROR;
+		&& (buffer[ERROR_OFFSET] != 0xFE))
+	{
+		return_value = IFD_COMMUNICATION_ERROR;
+		ccid_error(buffer[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
 	}
 
 	return return_value;
@@ -1108,7 +1089,7 @@ RESPONSECODE CmdXfrBlock(unsigned int reader_index, unsigned int tx_length,
 RESPONSECODE CCID_Transmit(unsigned int reader_index, unsigned int tx_length,
 	const unsigned char tx_buffer[], unsigned short rx_length, unsigned char bBWI)
 {
-	unsigned char cmd[10+CMD_BUF_SIZE];	/* CCID + APDU buffer */
+	unsigned char cmd[10+tx_length];	/* CCID + APDU buffer */
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 	status_t ret;
 
@@ -1118,7 +1099,8 @@ RESPONSECODE CCID_Transmit(unsigned int reader_index, unsigned int tx_length,
 		int r;
 
 		/* Xfr Block */
-		r = ControlUSB(reader_index, 0x21, 0x65, 0, tx_buffer, tx_length);
+		r = ControlUSB(reader_index, 0x21, 0x65, 0,
+			(unsigned char *)tx_buffer, tx_length);
 
 		/* we got an error? */
 		if (r < 0)
@@ -1140,8 +1122,8 @@ RESPONSECODE CCID_Transmit(unsigned int reader_index, unsigned int tx_length,
 
 		/* Xfr Block */
 		DEBUG_COMM2("chain parameter: %d", rx_length);
-		r = ControlUSB(reader_index, 0x21, 0x65, rx_length << 8, tx_buffer,
-			tx_length);
+		r = ControlUSB(reader_index, 0x21, 0x65, rx_length << 8,
+			(unsigned char *)tx_buffer, tx_length);
 
 		/* we got an error? */
 		if (r < 0)
@@ -1162,13 +1144,6 @@ RESPONSECODE CCID_Transmit(unsigned int reader_index, unsigned int tx_length,
 	cmd[8] = rx_length & 0xFF;	/* Expected length, in character mode only */
 	cmd[9] = (rx_length >> 8) & 0xFF;
 
-	/* check that the command is not too large */
-	if (tx_length > CMD_BUF_SIZE)
-	{
-		DEBUG_CRITICAL2("TX Length too big: %d", tx_length);
-		return IFD_NOT_SUPPORTED;
-	}
-
 	memcpy(cmd+10, tx_buffer, tx_length);
 
 	ret = WritePort(reader_index, 10+tx_length, cmd);
@@ -1211,6 +1186,9 @@ RESPONSECODE CCID_Receive(unsigned int reader_index, unsigned int *rx_length,
 			return IFD_COMMUNICATION_ERROR;
 		}
 
+		/* we need to store returned value */
+		*rx_length = r;
+
 		return IFD_SUCCESS;
 	}
 
@@ -1415,7 +1393,7 @@ static RESPONSECODE CmdXfrBlockAPDU_extended(unsigned int reader_index,
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 	unsigned char chain_parameter;
 	unsigned int local_tx_length, sent_length;
-	unsigned int local_rx_length, received_length;
+	unsigned int local_rx_length = 0, received_length;
 	int buffer_overflow = 0;
 
 	if (ICCD_B == ccid_descriptor->bInterfaceProtocol)
@@ -1764,8 +1742,6 @@ static RESPONSECODE T0ProcSW1(unsigned int reader_index,
 {
 	RESPONSECODE return_value = IFD_SUCCESS;
 	UCHAR tmp_buf[512];
-	unsigned char *rcv_buf_tmp = rcv_buf;
-	const unsigned int rcv_len_tmp = *rcv_len;
 	unsigned char sw1, sw2;
 
 	/* store the SW1 */
@@ -1791,17 +1767,9 @@ static RESPONSECODE T0ProcSW1(unsigned int reader_index,
 		in_buf = tmp_buf;
 	}
 	sw2 = *rcv_buf = *in_buf;
-	rcv_buf++;
-	in_buf++;
 	in_len--;
 	(*rcv_len)++;
 
-	if (return_value != IFD_SUCCESS)
-	{
-		rcv_buf_tmp[0] = rcv_buf_tmp[1] = 0;
-		*rcv_len = rcv_len_tmp;
-	}
-
 	DEBUG_COMM3("Exit: SW=%02X %02X", sw1, sw2);
 
 	return return_value;
@@ -1830,6 +1798,15 @@ static RESPONSECODE CmdXfrBlockCHAR_T0(unsigned int reader_index,
 	if (ICCD_A == ccid_descriptor->bInterfaceProtocol)
 	{
 		unsigned char pcbuffer[SIZE_GET_SLOT_STATUS];
+		unsigned int backup_len;
+
+		/* length is on 16-bits only
+		 * if a size > 0x1000 is used then usb_control_msg() fails with
+		 * "Invalid argument" */
+		if (*rcv_len > 0x1000)
+			*rcv_len = 0x1000;
+
+		backup_len = *rcv_len;
 
 		/* Command to send to the smart card (must be 5 bytes) */
 		memset(cmd, 0, sizeof(cmd));
@@ -1856,40 +1833,48 @@ static RESPONSECODE CmdXfrBlockCHAR_T0(unsigned int reader_index,
 		if (return_value != IFD_SUCCESS)
 			return return_value;
 
-		if ((0x20 != pcbuffer[0]) && (snd_len > 0))
+		if (0x10 == pcbuffer[0])
 		{
-			/* continue sending the APDU */
-			return_value = CCID_Transmit(reader_index, snd_len, snd_buf, 0, 0);
-			if (return_value != IFD_SUCCESS)
-				return return_value;
-		}
-		else
-		{
-			if ((0x20 == pcbuffer[0]) && (*rcv_len > 2))
-				/* we will just read SW1-SW2 */
-				*rcv_len = 2;
-
-			return_value = CCID_Receive(reader_index, rcv_len, rcv_buf, NULL);
-			if (return_value != IFD_SUCCESS)
-				DEBUG_CRITICAL("CCID_Receive failed");
-
-			return return_value;
+			if (snd_len > 0)
+			{
+				/* continue sending the APDU */
+				return_value = CCID_Transmit(reader_index, snd_len, snd_buf,
+					0, 0);
+				if (return_value != IFD_SUCCESS)
+					return return_value;
+			}
+			else
+			{
+				/* read apdu data */
+				return_value = CCID_Receive(reader_index, rcv_len, rcv_buf,
+						NULL);
+				if (return_value != IFD_SUCCESS)
+					return return_value;
+			}
 		}
 
-		/* wait for ready */
 		return_value = CmdGetSlotStatus(reader_index, pcbuffer);
 		if (return_value != IFD_SUCCESS)
 			return return_value;
 
-		if ((0x20 == pcbuffer[0]) && (*rcv_len > 2))
-			/* we will just read SW1-SW2 */
+		/* SW1-SW2 available */
+		if (0x20 == pcbuffer[0])
+		{
+			/* backup apdu data length */
+			/* if no data recieved before - backup length must be zero */
+			backup_len = (backup_len == *rcv_len) ? 0 : *rcv_len;
+
+			/* wait for 2 bytes (SW1-SW2) */
 			*rcv_len = 2;
 
-		/* read SW1-SW2 */
-		return_value = CCID_Receive(reader_index, rcv_len, rcv_buf, NULL);
-		if (return_value != IFD_SUCCESS)
-			DEBUG_CRITICAL("CCID_Receive failed");
+			return_value = CCID_Receive(reader_index, rcv_len,
+				rcv_buf + backup_len, NULL);
+			if (return_value != IFD_SUCCESS)
+				DEBUG_CRITICAL("CCID_Receive failed");
 
+			/* restore recieved length */
+			*rcv_len += backup_len;
+		}
 		return return_value;
 	}
 
@@ -2044,7 +2029,7 @@ static RESPONSECODE CmdXfrBlockTPDU_T1(unsigned int reader_index,
 RESPONSECODE SetParameters(unsigned int reader_index, char protocol,
 	unsigned int length, unsigned char buffer[])
 {
-	unsigned char cmd[10+CMD_BUF_SIZE];	/* CCID + APDU buffer */
+	unsigned char cmd[10+length];	/* CCID + APDU buffer */
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 
 	DEBUG_COMM2("length: %d bytes", length);
@@ -2056,10 +2041,6 @@ RESPONSECODE SetParameters(unsigned int reader_index, char protocol,
 	cmd[7] = protocol;	/* bProtocolNum */
 	cmd[8] = cmd[9] = 0; /* RFU */
 
-	/* check that the command is not too large */
-	if (length > CMD_BUF_SIZE)
-		return IFD_NOT_SUPPORTED;
-
 	memcpy(cmd+10, buffer, length);
 
 	if (WritePort(reader_index, 10+length, cmd) != STATUS_SUCCESS)
