@@ -1,7 +1,7 @@
 /*
     ifdhandler.c: IFDH API
     Copyright (C) 2003-2009   Ludovic Rousseau
-    Copyright (C) 2009-2012   Advanced Card Systems Ltd.
+    Copyright (C) 2009-2013   Advanced Card Systems Ltd.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -63,8 +63,8 @@ int PowerOnVoltage = VOLTAGE_5V;
 static int DebugInitialized = FALSE;
 
 // Card voltage and card type selection for ACR38U, ACR38U-SAM and SCR21U
-int ACR38CardVoltage = 0;
-int ACR38CardType = 0;
+BYTE ACR38CardVoltage = 0;
+DWORD ACR38CardType = 0;
 
 /* local functions */
 #if HAVE_DECL_TAG_IFD_POLLING_THREAD && !defined(TWIN_SERIAL) && defined(USE_USB_INTERRUPT)
@@ -623,7 +623,8 @@ EXTERNAL RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 						(ACS_ACR1281_DUAL_READER_QPBOC == readerID) ||
 						(ACS_ACR1281_DUAL_READER_BSI == readerID) ||
 						(ACS_ACR1281_1S_PICC_READER == readerID) ||
-						(ACS_ACR1251_1S_CL_READER == readerID))
+						(ACS_ACR1251_1S_CL_READER == readerID) ||
+						(ACS_ACR1251U_C == readerID))
 						*Value = 2;
 				}
 #endif
@@ -2108,19 +2109,6 @@ EXTERNAL RESPONSECODE IFDHICCPresence(DWORD Lun)
 		goto end;
 	}
 
-#ifndef __APPLE__
-	// ACR1222 or ACR1283 is required to read data from interrupt endpoint
-	if ((ACS_ACR85_PINPAD_READER_PICC == ccid_descriptor->readerID) ||
-		(ACS_ACR1222_DUAL_READER == ccid_descriptor->readerID) ||
-		(ACS_ACR1222_1SAM_DUAL_READER == ccid_descriptor->readerID) ||
-		(ACS_ACR1283_4S_CL_READER == ccid_descriptor->readerID) ||
-		(ACS_ACR1283_CL_READER == ccid_descriptor->readerID) ||
-		(ACS_ACR1283U_FW_UPGRADE == ccid_descriptor->readerID))
-	{
-		InterruptRead(reader_index, 10);
-	}
-#endif
-
 	/* save the current read timeout computed from card capabilities */
 	oldReadTimeout = ccid_descriptor->readTimeout;
 
@@ -2180,7 +2168,12 @@ EXTERNAL RESPONSECODE IFDHICCPresence(DWORD Lun)
 		}
 	}
 	else
+	{
+#ifndef __APPLE__
+		InterruptRead(reader_index, 10);
+#endif
 		return_value = CcidSlots[reader_index].pGetSlotStatus(reader_index, pcbuffer);
+	}
 
 	/* set back the old timeout */
 	ccid_descriptor->readTimeout = oldReadTimeout;
